#ifndef __WRAPPER_H
#define __WRAPPER_H

#include "config.h"
/* Kernel configuration issues */
#ifdef __KERNEL__

/* Some of these conditional statements were borrowed from Don Becker's
 * 3c509.c module code.
 */
#include <linux/config.h>
#include <linux/version.h>

/* define __SMP__ if we detect SMP kernel running at configure time */
#if defined(HAVE_SMP_KERNEL) && ! defined(__SMP__)
#define __SMP__
#endif
#if !defined(HAVE_SMP_KERNEL) && defined(__SMP__)
#undef __SMP__
#endif

/* define MODVERSIONS if we detect running kernel uses them at 
 * configure time
 */
#if defined(HAVE_MODVERSIONS) && !defined(MODVERSIONS)
#define MODVERSIONS
#endif
#if !defined(HAVE_MODVERSIONS) && defined(MODVERSIONS)
#undef MODVERSIONS
#endif

#if defined(MODVERSIONS) && defined(HAVE_LINUX_MODVERSIONS_H)
#include <linux/modversions.h>
#endif

/* This include moved down to ensure that it follows the inclusion of
 * modversions.h if it is used.  Otherwise we run into symbol problems
 * on new 2.4 kernels
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/uio.h>
#include <linux/socket.h>
#include <linux/ioctl.h>
#include <linux/stat.h>
#include <linux/sched.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/tcp.h>
#include <linux/slab.h>
#include <linux/signal.h>
#include <asm/unistd.h>
#include <asm/errno.h>
#include <asm/uaccess.h>

extern int Open(const char *pathname, int flags, int mode);
extern int Close(int fd);
extern ssize_t Read(int fd, void *buf, size_t count);
extern ssize_t Write(int fd, const void *buf, size_t count);
extern int Fstat(int filedes, struct stat *buf);
extern int Stat(const char *pathname, struct stat *buf);
extern int Lstat(const char* pathname, struct stat *buf);
extern int Fcntl(unsigned int fd, unsigned int cmd, unsigned long args);
extern int Socket(int domain,int type,int protocol);
extern int Bind(int sockfd,struct sockaddr *my_addr,int addrlen);
extern int Connect(int sockfd,struct sockaddr *serv_addr,int addrlen);
extern int Listen(int sockfd,int backlog);
extern int Accept(int sockfd,struct sockaddr *addr,int *addrlen);
extern int Getsockname(int sockfd,struct sockaddr *name,unsigned int *namelen);
extern int Getpeername(int sockfd,struct sockaddr *name,unsigned int *namelen);
extern int Socketpair(int domain,int type,int protocol,int *sv);
extern int Send(int sockfd,void *msg,int len,unsigned int flags);
extern int Recv(int sockfd,void *buf,int len,unsigned int flags);
extern int Shutdown(int sockfd,int how);
extern int Setsockopt(int sockfd,int level,int optname,void *optval,unsigned int optlen);
extern int Getsockopt(int sockfd,int level,int optname,void *optval,unsigned int *optlen);
extern int Select(int n, fd_set *readfds, fd_set *writefds, fd_set *except_fds, struct timeval *timeout);
extern int Sendfile(int out_fd, int in_fd, off_t *offset, size_t count);
extern int Readv(int fd,const struct iovec *vector, int count);
extern int Writev(int fd, const struct iovec *vector, int count);
extern int Poll(struct pollfd *ufds, unsigned int nfds, int timeout);

#ifndef PERROR
#define PERROR(format...)                                      \
  do {                                                            \
    printk("(%s, %d): ",  __FILE__, __LINE__);                    \
    printk(format);                                         \
  } while (0) ;
#endif

static __inline__ void *allocate_memory(size_t size)
{
	void *ptr;
	ptr = kmalloc(size, GFP_KERNEL);
	if(ptr) {
		memset(ptr, 0, size);
	}
	return ptr;
}

static __inline__ void release_memory(void *ptr)
{
	if(ptr) {
		kfree(ptr);
	}
	return;
}

/* x is negative */
#define RETURN_ERROR(x) \
do { \
	return (x);   \
}while(0); 

#else

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/poll.h>
#include <sys/select.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/sendfile.h>
#include <sys/time.h>
#include <string.h>
#include <strings.h>
#include <errno.h>

static __inline__ int Open(const char *pathname, int flags, int mode)
{
	return open(pathname, flags, mode);
}

static __inline__ int Close(int fd)
{
	return close(fd);
}

static __inline__ ssize_t Read(int fd, void *buf, size_t count)
{
	return read(fd, buf, count);
}


static __inline__ ssize_t Write(int fd, const void *buf, size_t count)
{
	return write(fd, buf, count);
}

static __inline__ int Fstat(int filedes, struct stat *buf)
{
	return fstat(filedes, buf);
}

static __inline__ int Stat(const char *pathname, struct stat *buf)
{
	return stat(pathname, buf);
}

static __inline__ int Lstat(const char* pathname, struct stat *buf)
{
	return lstat(pathname, buf);
}

static __inline__ int Fcntl(unsigned int fd, unsigned int cmd, unsigned long args)
{
	return fcntl(fd, cmd, args);
}

static __inline__ int Socket(int domain,int type,int protocol)
{
	return socket(domain,type,protocol);
}

static __inline__ int Bind(int sockfd,struct sockaddr *my_addr,int addrlen)
{
	return bind(sockfd,my_addr,addrlen);
}

static __inline__ int Connect(int sockfd,struct sockaddr *serv_addr,int addrlen)
{
	return connect(sockfd, serv_addr, addrlen);
}

static __inline__ int Listen(int sockfd,int backlog)
{
	return listen(sockfd, backlog);
}

static __inline__ int Accept(int sockfd,struct sockaddr *addr,int *addrlen)
{
	return accept(sockfd, addr, addrlen);
}

static __inline__ int Getsockname(int sockfd,struct sockaddr *name,unsigned int *namelen)
{
	return getsockname(sockfd, name, namelen);
}

static __inline__ int Getpeername(int sockfd,struct sockaddr *name,unsigned int *namelen)
{
	return getpeername(sockfd,name,namelen);
}

static __inline__ int Socketpair(int domain,int type,int protocol,int *sv)
{
	return socketpair(domain,type,protocol,sv);
}

static __inline__ int Send(int sockfd,void *msg,int len,unsigned int flags)
{
	return send(sockfd, msg, len,flags);
}

static __inline__ int Recv(int sockfd,void *buf,int len,unsigned int flags)
{
	return recv(sockfd, buf, len, flags);
}

static __inline__ int Shutdown(int sockfd,int how)
{
	return shutdown(sockfd, how);
}

static __inline__ int Setsockopt(int sockfd,int level,int optname,void *optval,unsigned int optlen)
{
	return setsockopt(sockfd, level, optname, optval, optlen);
}

static __inline__ int Getsockopt(int sockfd,int level,int optname,void *optval,unsigned int *optlen)
{
	return getsockopt(sockfd, level, optname, optval, optlen);
}

static __inline__ int Select(int n, fd_set *readfds, fd_set *writefds, fd_set *except_fds, struct timeval *timeout)
{
	return select(n, readfds, writefds, except_fds, timeout);
}

static __inline__ int Sendfile(int out_fd, int in_fd, off_t *offset, size_t count)
{
	return sendfile(out_fd, in_fd, offset, count);
}

static __inline__ int Readv(int fd,const struct iovec *vector, int count)
{
	return readv(fd, vector, count);
}

static __inline__ int Writev(int fd, const struct iovec *vector, int count)
{
	return writev(fd, vector, count);
}

static __inline__ int Poll(struct pollfd *ufds, unsigned int nfds, int timeout)
{
	return poll(ufds, nfds, timeout);
}

#ifndef PERROR
#define PERROR(format...)                                      \
  do {                                                            \
    printf("(%s, %d): ",  __FILE__, __LINE__);                    \
    printf(format);                                         \
	 fflush(stdout);                                               \
  } while (0) ;
#endif

static __inline__ void *allocate_memory(size_t size)
{
	void *ptr;
	ptr = malloc(size);
	if(ptr) {
		memset(ptr, 0, size);
	}
	return ptr;
}

static __inline__ void release_memory(void *ptr)
{
	if(ptr) {
		free(ptr);
	}
	return;
}

/* x is negative */
#define RETURN_ERROR(x) \
do { \
	errno = -(x); \
	return -1;   \
}while(0); 

#endif


#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 *
 * vim: ts=3
 */
